###############################################################################
#
#    NAME: kinitptype.tcl
#
#    PURPOSE: Init Kratos problem type
#
#    QUANTECH ATZ-DEVELOPMENT DEPARTMENT
#
#    AUTHOR : G. Socorro
#
#    CREATED AT: 01/02/10
#
#    HISTORY:
#
#     1.6-20/12/13-G. Socorro, update the proc SelectGIDBatFile to run convection-diffusion simulations
#     1.5-03/11/13-G. Socorro, add a call to the proc AssignSpecialBoundaries for the special case of DEM application
#     1.4-09/10/13-G. Socorro, add two new script file wkcfconvectiondiffusion.tcl wkcfdem.tcl
#     1.3-19/09/13-G. Socorro, modify the proc BeforeMeshGeneration to assign automatically triangle or quadrilateral element to the skin surfaces
#     1.2-18/06/13-G. Socorro, delete the proc kipt::NewGiDGroups
#     1.1-22/10/12-J. Garate, Support for new GiD Groups
#     1.0-08/10/12-J. Garate, Enable/disable kipt::NewGiDGroups
#     0.9-01/10/12-J. Garate, Enable/disable Curves Module
#     0.8-20/09/12-J. Garate, add Curves, Tables and Plotgraph source files
#     0.7-04/05/12-G. Socorro, add a new variable to control the group deletion (when exists from the problem type)  
#     0.6-03/05/12-G. Socorro, Delete all group identifier using ::KUtils::DeleteAllGroupIdentifier and 
#                              close all group window Cond_Groups window close
#     0.5-10/04/12-G. Socorro, load new script in the (wkcffluid.tcl, etc.)
#     0.4-02/04/12-J.Garate, icon path change to adapt to GiD Themes
#     0.3-29/03/12-G. Socorro, load the new kmprops scripts
#     0.2-22/06/11-G. Socorro, delete KPriv(release) and create KPriv(RDConfig) in the Kratos.tcl
#     0.1-01/02/10-G. Socorro, create the base source code
#
###############################################################################

#short alias
proc msg {message} {    
    WarnWinText $message
}

proc msgS {message} {    
    WarnWin $message
}

proc wa {message} {  
    WarnWinText $message
}

# kipt => kratos Init Problem Type 

#######################################################
### namespace Kratos ###
#######################################################

namespace eval kipt { } {
    variable ProgramName kratos
    variable VersionNumber ;#interface version, get it from xml to avoid duplication
    variable Web http://www.cimne.com/kratos
}


proc kipt::InitGlobalXMLVariables {} {
    global KPriv
    

}


proc kipt::InitGIDProject { dir } {
    
    global KPriv
    # Set dir to a global variable
    set KPriv(dir) $dir
    set KPriv(problemTypeDir) $dir
    
    
    # Read kratos.xml file
    if { [info procs ::ReadProblemtypeXml] == "" } {
        WarnWin [=  "This GiD version is too old, get the last available version"]
        return 1
    }
    set xmlfile [file join $dir kratos.xml]
    set data [ReadProblemtypeXml $xmlfile Infoproblemtype {Version MinimumGiDVersion}]
    # WarnWinText "dir:$dir\nxmlfile:$xmlfile data:$data"
    if { $data == "" } {
	if { [ file exists $xmlfile]} {
	    WarnWinText [= "Couldn't read file %s" $xmlfile]
	} else {
	    WarnWinText [= "Configuration file %s not found" $xmlfile]
	}
        return 1
    }
    array set problemtype_local $data
    set kipt::VersionNumber $problemtype_local(Version)
    
    # Check the required GiD version   
    if { [::GidUtils::VersionCmp $problemtype_local(MinimumGiDVersion)]  < 0 } {
        WarnWinText [= "Error: This interface requires GiD %s or later" $problemtype_local(MinimumGiDVersion)].
    }
    
    
    # Init some xml global variables
    ::kipt::InitGlobalXMLVariables            
    ::kipt::LoadSourceFiles $dir    
    
    
    
    set ptypeName [lindex [split $KPriv(problemTypeDir) "/"] end]
    set KPriv(pTypeName) [string map {".gid" ""} $ptypeName]
    
    # Set images directory
    set KPriv(imagesdir) [file join images]
    
    # Change system menu
    # First -> Wizard
    ::kwiz::CreateInitWizard

    
    # Maintain the problem type
    GiD_Set MaintainProblemTypeInNew 1
    return 0
}

proc kipt::EndGIDProject {} {    
    global KPriv    
    # Destroy all pre/post open window
    
    # Close Project Settings Window if it exists
    set w .gid.starwindow
    if {[winfo exists $w]} {
        ::kps::WindowbClose $w 
    }
}

proc kipt::LoadSourceFiles { dir } {
    
    # Load the application scripts for Kratos applications
    global KPriv
    
    # Load some packages
    set lib_paths [list]
    set lib_filenames [list]

   
    # For wizards
    lappend lib_paths [file join $dir scripts wizard]
    lappend lib_filenames {snitwiz.tcl}
    
    # Load KMProps
    lappend lib_paths [file join $dir scripts]
    lappend lib_filenames {initwindow.tcl winutils.tcl}
    
    #source them
    foreach lib_path $lib_paths filenames $lib_filenames {    
        foreach filename $filenames {
            set full_filename [file join $lib_path $filename]
            if { [catch {source $full_filename} msg ] } {            
                WarnWin [= "Error reading file '%s': %s" $full_filename $msg].
                return 1
            }
        }
    }

    
    return 0
}

